@@ -1,4 +1,4 @@
-# Copyright 1998-2019 Gentoo Authors
+# Copyright 1998-2020 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import unicode_literals
@@ -94,7 +94,7 @@ class bindbapi(fakedbapi):
 		# Selectively cache metadata in order to optimize dep matching.
 		self._aux_cache_keys = set(
 			["BDEPEND", "BUILD_ID", "BUILD_TIME", "CHOST", "DEFINED_PHASES",
-			"DEPEND", "EAPI", "HDEPEND", "IUSE", "KEYWORDS",
+			"DEPEND", "EAPI", "IUSE", "KEYWORDS",
 			"LICENSE", "MD5", "PDEPEND", "PROPERTIES",
 			"PROVIDES", "RDEPEND", "repository", "REQUIRES", "RESTRICT",
 			"SIZE", "SLOT", "USE", "_mtime_"
@@ -283,7 +283,7 @@ class bindbapi(fakedbapi):
 				logfile=settings.get('PORTAGE_LOG_FILE'),
 				scheduler=SchedulerInterface(loop))
 
-			extractor.start()
+			yield extractor.async_start()
 			yield extractor.async_wait()
 			if extractor.returncode != os.EX_OK:
 				raise PortageException("Error Extracting '{}'".format(pkg_path))
@@ -394,13 +394,13 @@ class binarytree(object):
 			self._pkgindex_aux_keys = \
 				["BASE_URI", "BDEPEND", "BUILD_ID", "BUILD_TIME", "CHOST",
 				"DEFINED_PHASES", "DEPEND", "DESCRIPTION", "EAPI",
-				"HDEPEND", "IUSE", "KEYWORDS", "LICENSE", "PDEPEND",
+				"IUSE", "KEYWORDS", "LICENSE", "PDEPEND",
 				"PKGINDEX_URI", "PROPERTIES", "PROVIDES",
 				"RDEPEND", "repository", "REQUIRES", "RESTRICT",
 				"SIZE", "SLOT", "USE"]
 			self._pkgindex_aux_keys = list(self._pkgindex_aux_keys)
 			self._pkgindex_use_evaluated_keys = \
-				("BDEPEND", "DEPEND", "HDEPEND", "LICENSE", "RDEPEND",
+				("BDEPEND", "DEPEND", "LICENSE", "RDEPEND",
 				"PDEPEND", "PROPERTIES", "RESTRICT")
 			self._pkgindex_header = None
 			self._pkgindex_header_keys = set([
@@ -417,7 +417,6 @@ class binarytree(object):
 				"DEFINED_PHASES"     : "",
 				"DEPEND"  : "",
 				"EAPI"    : "0",
-				"HDEPEND" : "",
 				"IUSE"    : "",
 				"KEYWORDS": "",
 				"LICENSE" : "",
